SuperStrict
Framework BRL.StandardIO
Import BRL.GLMax2D
Import BRL.LinkedList
Import BRL.Random

SeedRnd(MilliSecs())

Type TDrawable
	Global list:TList = CreateList()
	Field x:Int,y:Int,z:Int,rot:Int,img:TImage
	Function c(x:Int,y:Int,z:Int,rot:Int,img:TImage)
		Local tb:TDrawable = New TDrawable
		tb.x = x
		tb.y = y
		tb.z = z
		tb.rot = rot
		tb.img = img
		list.addLast(tb)
	End Function
	
	Method moveWithCamera()
		x = posX
		y = posY
		z = posZ+1
	End Method
	
	Method draw()
		Local ddif:Float = posZ-z
		SetScale(2^ddif,2^ddif)
		SetRotation(rot)
		SetColor(255,255+ddif*50,255+ddif*50)
		DrawImage(img,400-(posX-x)-2^ddif*136-136,300-(posY-y)-2^ddif*20-20)
		SetScale(1,1)
		SetRotation(0)
	End Method
	
	Function drawAll()
		For Local d:TDrawable = EachIn list
			If (d.z>posZ-3 And d.z-posZ<5) Then d.draw()
		Next
	End Function
	
	Function closestObj:TDrawable()
		Local cl:TDrawable
		Local cd:Float = 5000
		For Local d:TDrawable = EachIn list
			If Abs(d.z-posZ)<cd Then
				cl = d
				cd = Abs(d.z-posZ)
			End If
		Next
		Return cl
	End Function
End Type

Type TEndoSeg
	Global list:TList = CreateList()
	Global segnum:Int=0
	Field x:Int, y:Int, num:Int
	
	Function c(x:Int, y:Int)
		Local seg:TEndoSeg = New TEndoSeg
		seg.x = x
		seg.y = y
		seg.num = segnum
		segnum :+ 1
		list.addLast(seg)
	End Function
	
	Method draw()
		Local ddif:Float = posZ-num
		If (ddif<-4) Then Return
		Local size:Int = Min(1000*2^ddif,1500)
		SetColor(255,64+ddif*10,64+ddif*10)
		DrawOval(400-size/2-(posX+x)/Max(ddif,1),300-size/2+(posY+y)/Max(ddif,1),size,size)
	End Method
	
	Function drawAll()
		For Local es:TEndoSeg = EachIn list
			es.draw()
		Next
	End Function
End Type

Global posX:Float
Global posY:Float
Global posZ:Float

Graphics (800,600)

SetColor 0,0,0
DrawRect(0,0,272,22)
DrawRect(0,0,58,41)
SetColor 128,128,128
DrawRect(1,1,270,20)
DrawRect(1,1,56,39)
SetColor 0,0,0
For Local i:Int=4 To 56 Step 4
	DrawLine(1+i,20,1+i,40)
Next
Global toothbrushimg:TImage = CreateImage(272,58,1,DYNAMICIMAGE|MASKEDIMAGE)
GrabImage(toothbrushimg,0,0)
Cls
SetColor 128,128,128
DrawRect(22,0,174,13)
DrawLine(0,7,22,0)
DrawLine(0,7,22,13)
DrawLine(2,7,22,3)
DrawLine(2,7,22,10)
DrawLine(0,7,22,7)
SetColor 0,0,0
For Local i:Int=4 To 56 Step 4
	DrawLine(1+i,20,1+i,40)
Next
Global penimg:TImage = CreateImage(272,58,1,DYNAMICIMAGE|MASKEDIMAGE)
GrabImage(penimg,0,0)

generateLevel()

While Not KeyHit(KEY_ESCAPE)
	Cls
	TEndoSeg.drawAll()
	TDrawable.drawAll()
	SetColor(0,0,0)
	DrawText("Use L,R,U,D to navigate left, right, up and down",20,500)
	DrawText("Use Enter to move forward",20,520)
	DrawText("Use Backspace to backup",20,540)
	Local closest:TDrawable = TDrawable.closestObj()
	Local maxDistAllowed:Int = 150
	If (closest<>Null And Abs(closest.z-posZ)<2) Then
		DrawText("Hold G to grab that object and bring it out",20,560)
		If (KeyDown(KEY_G)) Then
			closest.moveWithCamera()
			maxDistAllowed = 80
		EndIf
	EndIf
	If (KeyDown(KEY_U)) Then posY:+10
	If (KeyDown(KEY_L)) Then posX:-10
	If (KeyDown(KEY_D)) Then posY:-10
	If (KeyDown(KEY_R)) Then posX:+10
	If (KeyDown(KEY_Enter)) Then posZ:+.03
	If (KeyDown(KEY_BACKSPACE)) Then posZ:-.03
	Local curr:TEndoSeg = TEndoSeg(TEndoSeg.list.ValueAtIndex(Int(posZ)))
	If (curr<>Null And (Abs(posX+curr.x)>maxDistAllowed Or Abs(posY+curr.y)>maxDistAllowed)) Then lost()
	Flip
Wend

Function lost()
	SetColor(0,0,0)
	DrawText("You poked through the stomach.",250,280)
	DrawText("Congratulations. You killed the patient!",200,300)
	DrawText("Nevertheless, you lost!",300,320)
	Flip
	WaitKey
	Flip
	generateLevel()
End Function

Function generateLevel()
	TEndoSeg.list = CreateList()
	TDrawable.list = CreateList()
	TEndoSeg.segnum=0
	posX=0
	posY=0
	posZ=1
	Local total:Int = Rand(40,50)
	Local lastX:Int=0
	Local lastY:Int=0
	Local tarX:Int=0
	Local tarY:Int=0
	For Local i:Int=0 To total
		TEndoSeg.c(lastX,lastY)
		If (i=total-5) Then
			Local r:Int = Rand(0,1)
			Select r
			Case 0
				TDrawable.c(lastX,lastY,i,Rand(0,360),toothbrushimg)
			Case 1
				TDrawable.c(lastX,lastY,i,Rand(0,360),penimg)
			EndSelect
		EndIf
		If Abs(lastX-tarX)<30 Then tarX = Rand(-400,400)
		If Abs(lastY-tarY)<30 Then tarY = Rand(-400,400)
		lastX :- Sgn(lastX-tarX)*Rand(0,30)
		lastY :- Sgn(lastY-tarY)*Rand(0,30)
	Next
End Function